unit AutoUpdate;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ItigProgressKnown, itignet, itigoptions, WinBinFile;

const wtCheckServer = 'CHECK_SERVER';
const wtCheckVersion = 'CHECK_VERSION';
const wtDownload = 'DOWNLOAD_APP';
const wtReportBug = 'REPORT_BUG';
const wtGetHelper = 'DOWNLOAD_HELPER';
const wtGetFile = 'DOWNLOAD_FILE';
const AUTOUPDATE_VERSION = 103;

type
  TAutoUpdate = class(TComponent)
  private
    { Private declarations }
    fApplicationName : String;
    fURL : String;
    fVersion : String;
    fVersionNo : Integer;
    fResourceName : String;

    fDebugMethod : TDebugMethod;
    fGetProxyMethod : TGetProxyFunction;

    Info : TItigNet;

    procedure CreateInfo;
    procedure StartUpgrade;
    function DownloadHelper(Helper : String) : Boolean;
    procedure CheckServerVersion;
  protected
    { Protected declarations }
  public
    DestinationFile : String;
    ResourceImage : String;
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure CheckForUpgrade(Verbose : Boolean);
    procedure PerformUpgrade;
    procedure SetVersion(NewVersion : String);
    procedure Debug(S : String);
    procedure GetFile(FileName : String);

    // ITIGNet RPC Calls
    procedure ActionCheckServerVersionRPC(Context : TRPCContext);
    procedure ActionCheckVersionRPC(Context : TRPCContext);
    procedure ActionDownloadRPC(Context : TRPCContext);
    procedure ActionDownloadHelperRPC(Context : TRPCContext);
    procedure ActionDownloadFileRPC(Context : TRPCContext);
  published
    { Published declarations }
    property ApplicationName : String read FApplicationName write FApplicationName;
    property URL : String read FURL write FURL;
    property DebugMethod : TDebugMethod read fDebugMethod write fDebugMethod;
    property Version : String read fVersion write SetVersion;
    property VersionNumber : Integer read fVersionNo write fVersionNo;
    property ResourceName : String read fResourceName write fResourceName;
    property OnGetProxy : TGetProxyFunction read fGetProxyMethod write fGetProxyMethod;
  end;

procedure Register;


implementation

uses ShellAPI, WinIOCTL;

type TIsDebuggerPresent = function : BOOL; stdcall;

function IsDebuggerPresent : BOOL;
var
   hModule : hInst;
//   Error : DWORD;
   JIsDebuggerPresent : TIsDebuggerPresent;
   P : Pointer;
begin
   Result := False;

   // see if we can get IsDebuggerPresent from kernel32.dll

   hModule := GetModuleHandle('kernel32.dll');
   if hModule = 0 then
   begin
      // wininet is not yet loaded...
      hModule := LoadLibrary('kernel32.dll');
      {if hModule = 0 then
      begin
         Error := GetLastError;
         //raise Exception.Create('Error loading Windows Internet Library.  ' + SysErrorMessage(Error));
      end;}
   end;

   if hModule <> 0 then
   begin
      P := GetProcAddress(hModule, 'IsDebuggerPresent');
      if P = nil then
      begin
         //raise Exception.Create('Could not find procedure ' + ProcName);
      end
      else
      begin
         JIsDebuggerPresent := p;
         Result := JIsDebuggerPresent;
      end;
   end;

end;

procedure Register;
begin
  RegisterComponents('Samples', [TAutoUpdate]);
end;

constructor TAutoUpdate.Create(AOwner: TComponent);
begin
   inherited;

   Info := nil;
end;

procedure TAutoUpdate.CreateInfo;
begin
   if not Assigned(Info) then
   begin
      Info := TItigNet.Create(0{Handle}, fDebugMethod, fGetProxyMethod);
      Info.RegisterRPC(wtCheckServer, ActionCheckServerVersionRPC);
      Info.RegisterRPC(wtCheckVersion, ActionCheckVersionRPC);
      Info.RegisterRPC(wtDownload, ActionDownloadRPC);
//      Info.RegisterRPC(wtReportBug, ActionReportBugRPC);
      Info.RegisterRPC(wtGetHelper, ActionDownloadHelperRPC);
      Info.RegisterRPC(wtGetFile, ActionDownloadFileRPC);
   end;
   Info.Option.SetURL(fURL);
   Info.Option.ApplicationName := fApplicationName;
   Info.Option.VersionNumber := fVersionNo;
   Info.Option.CurrentVersion := fVersion;
end;

procedure TAutoUpdate.Debug(S : String);
begin
   if Assigned(fDebugMethod) then
   begin
      fDebugMethod(s);
   end
end;

procedure TAutoUpdate.SetVersion(NewVersion : String);
begin
   fVersion := NewVersion;
   Inc(fVersionNo);
end;

procedure TAutoUpdate.CheckForUpgrade(Verbose : Boolean);
var
   Progress : TItigProgressKnown;
begin
   CreateInfo;

//   CheckServerVersion;
   CheckServerVersion;

   if Info.Option.ServerVersion >= AUTOUPDATE_VERSION then
   try
      // check server for new version
      Progress := TItigProgressKnown.Create(self);
      try
         Progress.AutoClose := True;
Debug('TAutoUpdate.CheckForUpgrade');
         Progress.Execute(wtCheckVersion, Info);
         if Info.GetResult = irOK then
         begin
            //MessageDlg('Success', mtInformation, [mbOK], 0);
            // check available version...
            if Info.Option.Available then
            begin
               // we should make a pretty form for this...
               if Length(Info.Option.Readme) > 0 then
               begin
                  MessageDlg(Info.Option.Readme, mtInformation, [mbOK], 0);
               end;
               if MessageDlg('A new version is available.  Do you want to upgrade now?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
               begin
                  // we need to save the proxy settings to the registry so that the helper can use them
                  Info.Option.SaveInternet;

                  StartUpgrade;
               end;
            end
            else
            begin
               if Verbose then
               begin
                  MessageDlg('There are no software updates available for this product at this time.', mtInformation, [mbOK], 0);
               end;
            end;
         end
         else
         begin
            //MessageDlg('The AutoUpdate procedure failed.', mtError, [mbOK], 0);
         end;
      finally
         Info.ClearResult;
         Progress.Free;
      end;
   finally
   end
   else
   begin
      if Verbose then
      begin
         MessageDlg('AutoUpdate Server Version does not match.  Please try manual upgrade', mtInformation, [mbOK], 0);
      end;
   end;
end;

procedure TAutoUpdate.CheckServerVersion;
var
   Progress : TItigProgressKnown;
begin
   CreateInfo;
   try
      // check server for new version
      Progress := TItigProgressKnown.Create(self);
      try
         Progress.AutoClose := True;
         Progress.Execute(wtCheckServer, Info);
         if Info.GetResult = irOK then
         begin
            // The number will be checked later
         end
         else
         begin
            //MessageDlg('The AutoUpdate procedure failed.', mtError, [mbOK], 0);
         end;
      finally
         Info.ClearResult;
         Progress.Free;
      end;
   finally
   end;
end;

procedure TAutoUpdate.GetFile(FileName : String);
var
   Progress : TItigProgressKnown;
begin
   CreateInfo;
   try
      // check server for new version
      Progress := TItigProgressKnown.Create(self);
      try
         Progress.AutoClose := True;
         Debug('TAutoUpdate.GetFile');
         Info.Option.DownloadedFile := FileName;
         Progress.Execute(wtGetFile, Info);
         if Info.GetResult = irOK then
         begin
            MessageDlg('The file was successfuly download.', mtInformation, [mbOK], 0);
            // downloaded...
         end
         else
         begin
//            MessageDlg('The file is not available for download.', mtInformation, [mbOK], 0);
         end;
      finally
         Info.ClearResult;
         Progress.Free;
      end;
   finally
   end;
end;

procedure ShellCopyFile(ParentWnd : HWND; Source : String; Dest : String);
var
   FileOp      : TSHFILEOPSTRUCT;
   FromFile    : PChar;
   ToFile      : PChar;
begin
   if FileExists(Dest) then
   begin
      DeleteFile(Dest);
   end;

   FromFile := StrAlloc(Length(Source) + 2);
   ZeroMemory(FromFile, Length(Source) + 2);
   Move(PChar(Source)^, FromFile^, Length(Source));

   ToFile := StrAlloc(Length(Dest) + 2);
   ZeroMemory(ToFile, Length(Dest) + 2);
   Move(PChar(Dest)^, ToFile^, Length(Dest));

   ZeroMemory(@FileOp, SizeOf(FileOp));
   FileOp.Wnd := ParentWnd;
   FileOp.wFunc := FO_COPY;
   FileOp.pFrom := FromFile;
   FileOp.pTo := ToFile;
   FileOp.fFlags := FOF_NOCONFIRMATION or FOF_SIMPLEPROGRESS;
   FileOp.lpszProgressTitle := 'Copying file.  Please wait...';

   SHFileOperation(FileOp);

   if FileOp.fAnyOperationsAborted then
   begin
      if not CopyFile(PChar(Source), PChar(Dest), False) then
      begin
         ShowMessage('Fallback copy failed with error=' + IntTOStr(GetLastError) + ' ' + SysErrorMessage(GetLastError));
      end;
   end;

   StrDispose(FromFile);
   StrDispose(ToFile);
end;

procedure TAutoUpdate.PerformUpgrade;
var
   Progress : TItigProgressKnown;
   StartupInf : STARTUPINFO;
   ProcessInf : PROCESS_INFORMATION;
   OldFileName : String;
   i : Integer;
begin
   CreateInfo;

   // we do this twice to make sure that the ssl options are set correctly?
   //CheckServerVersion;
   CheckServerVersion;
   if Info.Option.ServerVersion >= AUTOUPDATE_VERSION then
   try
      // check server for new version
      Progress := TItigProgressKnown.Create(self);
      try
         Progress.AutoClose := True;
         Progress.Execute(wtDownload, Info);
         if Info.GetResult = irOK then
         begin
            // copy the file...
            OldFileName := DestinationFile + '.old';
            if FileExists(OldFileName) then
            begin
               i := 1;
               while True do
               begin
                  OldFileName := DestinationFile + '.old.' + IntToStr(i);
                  if not FileExists(OldFileName) then
                  begin
                     break;
                  end;
                  Inc(i);
               end;
            end;
            ShellCopyFile(0, DestinationFile, OldFileName);
            ShellCopyFile(0, Info.Option.DownloadedFile, DestinationFile);

            // start the new application
            FillMemory(@StartupInf, sizeof(STARTUPINFO), 0);
            FillMemory(@ProcessInf, sizeof(PROCESS_INFORMATION), 0);

            StartupInf.cb := Sizeof(StartupInf);
            if CreateProcess( nil,
                           PChar(DestinationFile),
                           nil,
                           nil,
                           False,
                           0,
                           nil,
                           nil,
                           StartupInf,
                           ProcessInf)
               then
            begin
               CloseHandle(ProcessInf.hProcess);
               CloseHandle(ProcessInf.hThread);
               Application.Terminate;
            end
            else
            begin
               MessageDlg('Error restarting upgraded application', mtError, [mbOK], 0);
            end;
         end
         else
         begin
            MessageDlg('The AutoUpdate procedure failed.', mtError, [mbOK], 0);
         end;
      finally
         Info.ClearResult;
         Progress.Free;
      end;
   finally
   end
   else
   begin
      MessageDlg('AutoUpdate Server Version does not match.  Please try manual upgrade', mtInformation, [mbOK], 0);
   end;
end;

function TAutoUpdate.DownloadHelper(Helper : String) : Boolean;
var
   Progress : TItigProgressKnown;
   F : TBinaryFile;
begin
   Result := false;
   CreateInfo;
   Info.Option.DownloadedFile := Helper;
   try
      // check server for new version
      Progress := TItigProgressKnown.Create(self);
      try
         Progress.AutoClose := True;
         Progress.Execute(wtGetHelper, Info);
         if Info.GetResult = irOK then
         begin
            //MessageDlg('Success', mtInformation, [mbOK], 0);
            // check the file size...
            f := TBinaryFile.Create;
            try
               f.Assign(Helper);
               if f.FileSize > 0 then
               begin
                  Result := True;
               end
               else
               begin
                  f.Delete;
               end;
            finally
               f.Free;
            end;
         end
         else
         begin
            MessageDlg('The AutoUpdate procedure failed.', mtError, [mbOK], 0);
            try
               DeleteFile(Helper);
            except
            end;
         end;
      finally
         Info.ClearResult;
         Progress.Free;
      end;
   finally
   end;
end;

procedure TAutoUpdate.StartUpgrade;
var
   Helper : String;
   Command : String;
   StartupInf : STARTUPINFO;
   ProcessInf : PROCESS_INFORMATION;
   Error : DWORD;
begin
   // Always Download a new helper app
   Helper := ExtractFilePath(Application.ExeName) + 'autohelp.exe';

   if FileExists(Helper) then
   begin
      DeleteFile(Helper);
   end;


   if DownloadHelper(Helper) then
   begin

      FillMemory(@StartupInf, sizeof(STARTUPINFO), 0);
      FillMemory(@ProcessInf, sizeof(PROCESS_INFORMATION), 0);

      StartupInf.cb := Sizeof(StartupInf);
      // Start the helper and then kill ourself...
      // do we need to check security....?
      Command := Helper + ' ' + fURL + ' ' + fApplicationName + ' upgrade "' + Application.ExeName + '" ' + ResourceName;
      if IsDebuggerPresent then
      begin
         if MessageDlg('This process is currently being debugged.  Do you wish to create the child process?', mtError, [mbYes, mbNo], 0) = mrNo then
         begin
            exit;
         end;
      end;

      if CreateProcess( nil,
                     PChar(Command),
                     nil,
                     nil,
                     False,
                     0,
                     nil,
                     nil,
                     StartupInf,
                     ProcessInf)
         then
      begin
         CloseHandle(ProcessInf.hProcess);
         CloseHandle(ProcessInf.hThread);
         Application.Terminate;
      end
      else
      begin
         Error := GetLastError;
         MessageDlg('Error ' + IntToStr(Error) + ' starting upgrade procedure. ' + SysErrorMessage(Error), mtError, [mbOK], 0);
      end;
   end
   else
   begin
      // we could not download the helper so we can't upgrade
   end;
end;

procedure TAutoUpdate.ActionCheckServerVersionRPC(Context : TRPCContext);
var
   i : Integer;
begin
   Context.AddKeyValue('action', 'server_check');

   Context.Post.ScriptName := Context.Network.Option.GetScriptName;
   Context.Execute('');
   Debug(Context.Results.Text);

   Context.Network.Option.ServerVersion := 0;
   for i := 0 to Context.Keys.Count - 1 do
   begin
      if Context.Keys[i] = 'version' then
      begin
         Context.Network.Option.ServerVersion := StrToIntDef(Context.Values[i], 0);
      end;
   end;
end;

procedure TAutoUpdate.ActionCheckVersionRPC(Context : TRPCContext);
var
   i : Integer;
   Readme : String;
begin
   Context.AddKeyValue('action', 'check');
   Context.AddKeyValue('application', Context.Network.Option.ApplicationName);
   Context.AddKeyValue('version', Context.Network.Option.CurrentVersion);
   Context.AddKeyValue('versionno', IntToStr(Context.Network.Option.VersionNumber));

   Context.Post.ScriptName := Context.Network.Option.GetScriptName;
   Context.Execute('');
   Debug(Context.Results.Text);

   Context.Network.Option.Available := False;
   for i := 0 to Context.Keys.Count - 1 do
   begin
      if Context.Keys[i] = 'available' then
      begin
         if Context.Values[i] = 'true' then
         begin
            Context.Network.Option.Available := True;
         end;
      end;
      if Context.Keys[i] = 'readme' then
      begin
         if Length(Readme) > 0 then
         begin
            Readme := Readme + #10;
         end;
         Readme := Readme + Context.Values[i];
      end;
   end;

   Context.Network.Option.Readme := Readme;
end;

procedure TAutoUpdate.ActionDownloadRPC(Context : TRPCContext);
begin
   Context.AddKeyValue('action', 'download');
   Context.AddKeyValue('application', Context.Network.Option.ApplicationName);

   Context.Network.Option.DownloadedFile := Context.Network.Option.GetTempPath + Context.Network.Option.ApplicationName + '.autoupgrade';
   Context.ReceiveFile('', Context.Network.Option.DownloadedFile);
end;

procedure TAutoUpdate.ActionDownloadHelperRPC(Context : TRPCContext);
begin
   Context.AddKeyValue('action', 'gethelper');
   Context.AddKeyValue('application', Context.Network.Option.ApplicationName);

   Context.ReceiveFile('', Context.Network.Option.DownloadedFile);
end;

procedure TAutoUpdate.ActionDownloadFileRPC(Context : TRPCContext);
begin
   Context.AddKeyValue('action', 'getfile');
   Context.AddKeyValue('application', Context.Network.Option.ApplicationName);
   Context.AddKeyValue('filename', Context.Network.Option.DownloadedFile);
   Context.ReceiveFile('', Context.Network.Option.DownloadedFile);
end;

end.
